\pede.xpl	11-Jun-2017
\Centipede Arcade Game
\ by Richard Ottosen, Bill Bailey and Loren Blaney
\
\Revisions:
\04-Nov-1981: Original ran on Apple II in text mode.
\20-Nov-2003: Added graphics and modified for PC, LB.
\11-Jul-2012: Cleaned up code.
\11-Jun-2017: Modified for Raspberry Pi, LB.
\
\The 320x200 graphic screen (mode $13) is is divided into 8x8-pixel cells,
\ giving a column and line (Width and Height) of 40x25. All graphic objects
\ (sprites) are mapped into one (or two) of these 8x8 cells. A 40x24 play
\ field array (Field) represents all but the bottom line of the screen.

def	Width= 40, Height= 25,	\screen dimensions in 8x8 pixel cells
	Length= 12,		\initial number of segments in centipede
	\Warning: Width and Length can't be changed unless PlayRound is changed
	Tail= Length-1,		\index to centipede's last segment
	HeadTime= 30,		\delay for injecting heads near bottom
	MushIni= Width*Height/24, \initial number of mushrooms
	PlayIni= 3,		\initial number of players
	SpeedIni= 9,		\initial speed of game (60ths of a second)
	SpeedMax= 5,		\limit maximum speed after 40000 points
	SpiLimIni= Height-12;	\initial spider limit (line)

def	Right= Width-1,		\right-most (8-pixel wide) column of screen
	Center= Width/2,	\center column of screen
	Bottom= Height-1,	\bottom-most line number of screen (for scores)
	BotLimit= Bottom-1,	\lower limit of all objects on play field
	TopLimit= BotLimit-4;	\upper limit of player & upward moving centipede

def	Blank, Head, Segment,	\objects that go into play field
	Player, Shell,
	Mush, Mush3, Mush2, Mush1,
	SpiderL, SpiderR, Flea,
	Explo;

def	Esc= $1B;		\Esc key exits program

def	Black, Blue, Green, Cyan, Red, Magenta, Brown, White,  \attribute colors
	Gray, LBlue, LGreen, LCyan, LRed, LMagenta, Yellow, BWhite;

int	Armed,			\flag: player is armed with shell
	Bitten,			\flag: something bit the player (he's dead)
	FleaDrop,		\flag: flea is dropping (watch out!)
	FleaMad,		\flag: flea is mad cuz it's been shot once
	FleaX, FleaY,		\flea's field coordinates
	FleaWas,		\what the flea moved on top of (restores mushes)
	FullMoon,		\timer for injecting heads near bottom
	II,			\scratch for Main
	Players,		\total number of players remaining
	PlayX, PlayY,		\player's field coordinates
	Score,			\current total score
	HitBottom,		\flag: centipede has reached BotLimit
	HiScore,		\highest score in game series
	SegCnt,			\current number of live segments (incl. heads)
	ShellX, ShellY,		\shell's field coordinates
	SpiBias,		\pushes spider right or left (+/-1)
	SpiLimit,		\upper limit of spider (line)
	SpiX, SpiY,		\spider's field coordinates (left side)
	SpiXDir, SpiYDir,	\direction spider is currently moving
	Vol,			\flag: sound volume is on
	Wave;			\number of wave being played (starts at 0)

char	Field(Width, Height-1);	\the play field

int	Seg(Length),		\centipede segment type: Head, Segment, or Blank
	SegX(Length),		\segment's (or head's) field coordinates
	SegY(Length),
	SegXDir(Length),	\X direction of a segment (left=-1, right=+1)
	SegYDir(Length);	\Y direction of a segment (up=-1, down=+1)

\sprites (8x8 pixel graphic images)
char	SpBlank(2+8*8),		\first 2 bytes hold width and height in pixels
	SpHead(8, 2+8*8),	\centipede's head (array of 8 for leg positions)
	SpSeg(8, 2+8*8),
	SpPlay(2+8*8),
	SpShell(2+8*8),
	SpMush(2+8*8),
	SpMush3(2+8*8),
	SpMush2(2+8*8),
	SpMush1(2+8*8),
	SpSpiL0(2+8*8),
	SpSpiR0(2+8*8),
	SpSpiL1(2+8*8),
	SpSpiR1(2+8*8),
	SpFlea0(2+8*8),
	SpFlea1(2+8*8),
	SpExplo0(2+8*8),
	SpExplo1(2+8*8);

char	Palette(256*3);		\color register array

\=============================== MOUSE ROUTINES ================================

int	MouseX, MouseY,		\current mouse coordinates (pixels)
	MouseB;			\mouse buttons: bit 0 set = left down, 1 = right



func	GetMousePosition(N); \Return position of specified mouse coordinate
int	N;	\0 = X coordinate; 1 = Y coordinate
int	Mouse;
[Mouse:= GetMouse;
MouseX:= Mouse(0);  MouseY:= Mouse(1);
return if N then MouseY else MouseX;
];	\GetMousePosition



func	GetMouseButton;		\Returns non-zero if either mouse button is down
int	Mouse;			\ also checks for keyboard commands
[if KeyHit then
	case GetKey of
	 ^S,^s:	Vol:= ~Vol;		\toggle sound off/on
	  Esc:	[SetVid(3);  exit]	\restore normal text mode and exit
	other	[];
Mouse:= GetMouse;
MouseB:= Mouse(2);
return MouseB;
];	\GetMouseButton



proc	MoveMouse(X, Y);	\Move mouse pointer to X,Y
int	X, Y;
begin
\\CallInt($33, $0004, 0, X, Y);
];



cond false;
proc	MoveMouse;		\Move mouse pointer
int	Mouse;
[Mouse:= GetMouse;
MouseX:= Mouse(0);  MouseY:= Mouse(1);  MouseB:= Mouse(2);
if MouseX#MouseX0 ! MouseY#MouseY0 then			\mouse moved
	[DrawSprite(MouseX0, MouseY0, MouseBg, false);	\restore backgnd image
	MouseX0:= MouseX;  MouseY0:= MouseY;		\new backgnd location
	GetSprite(MouseX0, MouseY0, MouseW, MouseH, MouseBg); \save new backgnd
	DrawPointer(MouseX, MouseY);			\show pointer at new loc
	];
];	\MoveMouse
cond true;

\===============================================================================

proc	Delay(N);		\Delay N sixtieths of a second
int	N;
int	I;
for I:= 1 to N do WaitForVSync;



proc	Noise(Cy, Per);		\Emit a sound
int	Cy, Per;		\number of cycles, period in 10 ns intervals
int	Cnt, I;
int	T0;
[\Delay approximately the same as the original Apple II Sound intrinsic
for I:= 1 to Cy do DelayUS(Per*10);
];	\Noise



proc	PlaySound(Str);		\Play sound file unless Vol is turned off, S key
char	Str;
[if Vol then PlaySoundFile(Str);
];	\PlaySound

\===============================================================================

proc	CycleColors;		\Cycle 32 shades of 7 colors
int	I, J, Reg,
	R, G, B,
	R0, G0, B0;
[for I:= 0 to 31 do			\for 32 shades...
	[Reg:= (I+32)*3;		\save first color
	R0:= Palette(Reg+0);
	G0:= Palette(Reg+1);
	B0:= Palette(Reg+2);
	for J:= 1 to 6 do		\shift 6 colors down
		[Reg:= (I+(J+1)*32)*3;
		R:= Palette(Reg+0);
		G:= Palette(Reg+1);
		B:= Palette(Reg+2);
		Reg:= (I+J*32)*3;
		Palette(Reg+0):= R;
		Palette(Reg+1):= G;
		Palette(Reg+2):= B;
		];
	Reg:= (I+7*32)*3;		\set last color with first color
	Palette(Reg+0):= R0;
	Palette(Reg+1):= G0;
	Palette(Reg+2):= B0;
	];
for I:= 0 to 255 do			\set color registers from Palette
	SetPalette(I, Palette(I*3+0), Palette(I*3+1), Palette(I*3+2));
];	\CycleColors



proc	DrawSprite(X0, Y0, S, Mirror);		\Draw a sprite on the screen
int	X0, Y0;		\pixel coordinates of upper-left corner of sprite
char	S;		\address of sprite data
int	Mirror;		\flag: mirror image: reverse left and right
int	X, Y, K, W, H;
[W:= S(0);		\get width and height (in pixels)
 H:= S(1);
K:= 2;
for Y:= Y0 to Y0+H-1 do
	[if Mirror then
	    for X:= X0+W-1 downto X0 do
		[Point(X, Y, S(K));  K:= K+1]
	else
	    for X:= X0 to X0+W-1 do
		[Point(X, Y, S(K));  K:= K+1];
	];
];	\DrawSprite



proc	DrawObj(X, Y, Obj, Ex);	\Draw object at Field coordinates
int	X, Y,		\8x8 pixel cell coordinates
	Obj,		\object to draw
	Ex;		\extra modifier (segment direction, etc.)
int	Mirror;
char	Sp;
[Mirror:= false;
case Obj of
  Head:		[Sp:= @SpHead(Ran(8), 0);  Mirror:= Ex];
  Segment:	[Sp:= @SpSeg(Ran(8), 0);   Mirror:= Ex];
  Player:	Sp:= SpPlay;
  Shell:	Sp:= SpShell;
  Mush:		Sp:= SpMush;
  Mush3:	Sp:= SpMush3;
  Mush2:	Sp:= SpMush2;
  Mush1:	Sp:= SpMush1;
  SpiderL:	Sp:= if Ex then SpSpiL0 else SpSpiL1;
  SpiderR:	Sp:= if Ex then SpSpiR0 else SpSpiR1;
  Flea:		[Sp:= if Y&1 then SpFlea0 else SpFlea1;  Mirror:= Ex];
  Explo:	Sp:= if Ran(2) then SpExplo0 else SpExplo1
other		Sp:= SpBlank;
DrawSprite(X<<3, Y<<3, Sp, Mirror);
];	\DrawObj

\-------------------------------------------------------------------------------

proc	ShowPlayers;		\Show the waiting players
int	I;
[for I:= 2 to 12 do
	DrawObj(I+1, Bottom, if I<=Players then Player else Blank, 0)
];	\ShowPlayers



proc	SetUp(X, Y, Obj, Ex);	\Set object into Field array and display it
int	X, Y, Obj, Ex;
[if X>=0 & X<=Right & Y>=0 & Y<Height then	\X,Y coordinate is on the screen
	[Field(X, Y):= Obj;
	DrawObj(X, Y, Obj, Ex);
	];
];	\SetUp

\-------------------------------------------------------------------------------

proc	StartSpider;		\Start the spider just off the screen
[SpiBias:= Ran(2);		\randomly pick left or right direction of motion
if SpiBias = 0 then SpiBias:= -1;	\+1 or -1
SpiX:= if SpiBias < 0 then Right+Width/2 else -Width/2;
SpiXDir:= SpiBias;
];	\StartSpider

\-------------------------------------------------------------------------------

proc	BumpScore(Points);	\Increment Score by Points and display it
int	Points;
int	I;
[if Score/10000 < (Score+Points)/10000 then
	[Players:= Players+1;	\add a new player every 10000 points
	ShowPlayers;
	Delay(2);	\pause between last score's sound and new player sound
	PlaySound("Centipede_Tone01.wav");
	for I:= 0 to 1 do [Noise(25, 191);  Noise(25, 143)];
	PlaySound("Centipede_Tone02.wav");
	];
Score:= Score + Points;
Cursor(Width-18, Bottom);  Attrib(Yellow);  IntOut(6, Score);
];	\BumpScore

\-------------------------------------------------------------------------------

proc	ScoreHit;		\Bump score if shell hit something
int	I, Points;



proc	FlashNum(X, Y, N);	\Flash number of points (N) on screen at X,Y
int	X, Y, N;
int	J;
[if X > Right-2 then X:= Right-2;		\don't go off edge (spider)
if X < 0 then X:= 0;
Cursor(X, Y);  Attrib(White);  IntOut(6, N);
PlaySound("Centipede_Kill.wav");
for J:= 100 to 150 do Noise(3, Ran(J));		\make splat sound
Delay(6);
SetUp(X,   Y, Blank, 0);	\remove anything the score may have overwritten
SetUp(X+1, Y, Blank, 0);
SetUp(X+2, Y, Blank, 0);
];	\FlashNum



proc	ScoreSeg;		\Hit a segment or head
int	J;
[loop for J:= 0 to Tail do			\find the hit segment (or head)
	if ShellX=SegX(J) & ShellY=SegY(J) & Seg(J)#Blank then quit;
SetUp(SegX(J), SegY(J), Blank, 0);		\remove it
if Seg(J) = Head then SetUp(SegX(J)+SegXDir(J), SegY(J), Mush, 0)
	\(make it a mushroom shifted over one position)
else SetUp(SegX(J), SegY(J), Mush, 0);
Seg(J):= Blank;
\Grow a new head at the next segment
if J<Tail & Seg(J+1)=Segment then Seg(J+1):= Head;
SegCnt:= SegCnt -1;
];	\ScoreSeg



[	\ScoreHit
Points:= 0;			\default points
case Field(ShellX, ShellY) of
  Mush:		[for I:= 2 to 50 do Noise(2, Ran(I));
		SetUp(ShellX, ShellY, Mush3, 0);
		];
  Mush3:	[for I:= 2 to 50 do Noise(2, Ran(I));
		SetUp(ShellX, ShellY, Mush2, 0);
		];
  Mush2:	[for I:= 2 to 50 do Noise(2, Ran(I));
		SetUp(ShellX, ShellY, Mush1, 0);
		];
  Mush1:	[Points:= 1;
		for I:= 2 to 50 do Noise(2, Ran(I));
		SetUp(ShellX, ShellY, Blank, 0);
		];
  Segment:	[Points:= 10;
		PlaySound("Centipede_Kill.wav");
		for I:= 50 to 100 do Noise(3, Ran(I));
		ScoreSeg;
		];
  Head:		[Points:= 100;
		PlaySound("Centipede_Kill.wav");
		for I:= 50 to 100 do Noise(3, Ran(I));
		ScoreSeg;
		];
  SpiderL, SpiderR:
		[case (PlayY-SpiY+1) >> 1 of
		  1:	Points:= 900;
		  2:	Points:= 600
		other	Points:= 300;
		FlashNum(SpiX, SpiY, Points);
		SetUp(ShellX-1, ShellY, Blank, 0);	\remove dead spider
		SetUp(ShellX, ShellY, Blank, 0);
		SetUp(ShellX+1, ShellY, Blank, 0);
		StartSpider;
		];
  Flea:		[if FleaMad then
			[Points:= 200;
			for I:= 100 to 150 do Noise(3, Ran(I));
			SetUp(ShellX, ShellY, Blank, 0);
			FleaDrop:= false;
			]
		else	FleaMad:= true;
		]
other		[];
BumpScore(Points);
];	\ScoreHit

\-------------------------------------------------------------------------------

proc	KillPlayer;		\Something bit the player (i.e. you're dead)
int	I, J, X, Y;
[PlaySound("Centipede_Death.wav");
for J:= 0 to 8 do		\you're dead, player (awful noise)
	[			\make sure player, and possibly spider is gone
	SetUp(PlayX-1, PlayY, Explo, 0);
	SetUp(PlayX,   PlayY, Explo, 0);
	SetUp(PlayX+1, PlayY, Explo, 0);
	for I:= 0 to 10 do Noise(1, Ran(J*150));
	];
SetUp(PlayX-1, PlayY, Blank, 0); \remove dead player from field
SetUp(PlayX,   PlayY, Blank, 0);
SetUp(PlayX+1, PlayY, Blank, 0);
Players:= Players -1;		 \one less player

for X:= 0 to Right do		\restore shot-up mushrooms
    for Y:= 0 to BotLimit do
	case Field(X, Y) of
	  Mush1, Mush2, Mush3:
		[BumpScore(5);
		SetUp(X, Y, Mush, 0);
		PlaySound("Centipede_Bonus.wav");
		for I:= 2 to 3 do Noise(2, Ran(I));
		Delay(12);
		]
	other	[];

Delay(40);			\pause for effect
];	\KillPlayer

\-------------------------------------------------------------------------------

proc	MoveFlea;		\Move the flea
int	X, Y, I;
[if Wave = 0 then return;	\(give the player a break to start with)
if FleaDrop then
	[if FleaY < BotLimit then
		[if FleaWas = Blank then		\leave mushrooms in wake
		   SetUp(FleaX, FleaY, if Ran(Height/6) then Blank else Mush, 0)
		\replace what the flea hopped on except if it is a moving object
		else	case FleaWas of
			  Head, Segment, Player, Shell, SpiderL, SpiderR:
				SetUp(FleaX, FleaY, Blank, 0)
			other SetUp(FleaX, FleaY, FleaWas, 0);
		FleaY:= FleaY+1;
		FleaWas:= Field(FleaX, FleaY);	\save what the flea is on
		SetUp(FleaX, FleaY, Flea, FleaX<Center);
		if FleaWas = Player then Bitten:= true
		else if FleaWas = Shell then
			[ScoreHit;  Armed:= true];
		if FleaMad then Noise(5, FleaY*3+20)
		else		Noise(2, FleaY*30+200);
		]
	else	[FleaDrop:= false;		\hit bottom
		SetUp(FleaX, FleaY, Blank, 0);	\eradicate flea
		];
	]
else	[\Count the objects (mushrooms) in the bottom 5 rows
	I:= 0;
	for Y:= TopLimit to BotLimit do
	    for X:= 0 to Right do
		if Field(X,Y) # Blank then I:= I+1;
	\If the objects (mushrooms) are less than threshold then drop the flea
	if I < 5+Wave/2 then
		[FleaDrop:= true;
		FleaMad:= false;
		FleaX:= Ran(Width);  FleaY:= 0;
		FleaWas:= Field(FleaX, FleaY);
		];
	];
];	\MoveFlea

\-------------------------------------------------------------------------------

proc	MoveSpider;		\Bounce the spider around
int	X, Y;
[X:= SpiX;  Y:= SpiY;

if X<=0 ! X>=Right then				\spider enters at the top line
	[Y:= SpiLimit;  SpiXDir:= SpiBias;  SpiYDir:= 1];

if Ran(10) = 0 then SpiXDir:= Ran(3)-1+SpiBias;	\10% of the time change horz dir
if SpiXDir < 0 then X:= SpiX -1			\1/3 of time move horizontally
else if SpiXDir > 0 then X:= SpiX+1;		\2 1 0   1 0-1   0-1-2   2 1 0

if Ran(10) = 0 then SpiYDir:= Ran(5) - 2;	\10% of the time change vert dir
if SpiYDir <= 0 then Y:= Y -1			\SpiYDir = -2, -1, 0, 1, or 2
else if SpiYDir > 0 then Y:= Y+1;

if Y > BotLimit then
	[Y:= BotLimit-1;  SpiYDir:= -SpiYDir]
else if Y < SpiLimit then
	[Y:= SpiLimit+1;  SpiYDir:= 1];

\Erase what the spider was on
\(spider eats mushrooms; moving objects restore themselves)
SetUp(SpiX,   SpiY, Blank, 0);
SetUp(SpiX+1, SpiY, Blank, 0);

if SpiX<0 & X>=0  !  SpiX>=Width & X<Width then	\start sound if entering screen
	PlaySound("Centipede_SpiderLoop.wav");

SpiX:= X;  SpiY:= Y;				\actually move the spider

SetUp(SpiX, SpiY, SpiderL, SpiYDir<=0);
SetUp(SpiX+1, SpiY, SpiderR, SpiYDir<=0);

if SpiX+1>=0 & SpiX<=Right then
	Noise(2, (SpiY-SpiLimit)*10+100);

if SpiY = PlayY then
    if SpiX=PlayX ! SpiX+1=PlayX then
	Bitten:= true;

\If spider went 1/2 way off the screen then reverse its direction
if SpiX > Right+Width/2 then SpiBias:= -1
else if SpiX < -Width/2 then SpiBias:= 1;
];	\MoveSpider

\-------------------------------------------------------------------------------

proc	InjectHead;		\Inject a new head at bottom to prevent boredom
int	I, Zombie;
[if FullMoon > HeadTime then		\the zombies emerge
    [\If a head reached the bottom (and is going back up) then inject a head.
    for I:= 0 to Tail do
	if SegYDir(I)<0 & Seg(I)#Blank then HitBottom:= true;
    Zombie:= -1;			\start with no dead bodies
    if HitBottom then
	for I:= 0 to Tail do		\find a dead segment to revive
	    if Seg(I) = Blank then Zombie:= I;
    if Zombie >= 0 then			\we found a live one (sort of)
	[Seg(Zombie):= Head;
	SegXDir(Zombie):= if Ran(2) then 1 else -1;
	SegYDir(Zombie):= 1;
	SegX(Zombie):= if SegXDir(Zombie) = 1 then 0 else Right;
	SegY(Zombie):= TopLimit -1;
	SegCnt:= SegCnt+1;		\the zombie flesh lives!
	];
    FullMoon:= 0;			\now for a period of darkness
    ]
else FullMoon:= FullMoon+1;
];	\InjectHead

\-------------------------------------------------------------------------------

proc	MoveCent;		\Move the centipede
int	I, X0, Y0, X1, Y1;

	proc	Turn;		\Turn the segment down (or up) and around
	[\Move the segment down (or up) one row and reverse horizontal direction
	Y1:= Y0 + SegYDir(I);			\make tenative vertical move
	if Y1 > BotLimit then SegYDir(I):= -1	\if too far then reverse dir
	else if Y1 < TopLimit then SegYDir(I):= +1;
	SegY(I):= Y0 + SegYDir(I);		\make actual vertical move
	SegXDir(I):= -SegXDir(I);
	];	\Turn

[for I:= 0 to Tail do
    [if Seg(I) # Blank then			\don't move dead segments
	[X0:= SegX(I);  Y0:= SegY(I);		\get initial position
	 X1:= X0 + SegXDir(I);			\make tenative horizontal move

	if X1 < 0 then Turn			\if screen limit then turn
	else if X1 > Right then Turn
	else	case Field(X1, Y0) of		\bounce segment off mushrooms
		  Blank, Player, Shell, SpiderL, SpiderR, Flea:
			SegX(I):= X1		\ and other segments
		other Turn;

	SetUp(X0, Y0, Blank, 0);		\remove segment from old loc
	SetUp(SegX(I), SegY(I), Seg(I), SegXDir(I)<0); \ & display it in new loc

	if SegX(I)=PlayX & SegY(I)=PlayY then Bitten:= true;
	];
    ];
];	\MoveCent

\-------------------------------------------------------------------------------

proc	MoveShell;		\Move the shell if it has been fired
[if Armed then
	[if GetMouseButton then			\fire!
		[PlaySound("Centipede_Fire2.wav");
		ShellX:= PlayX;  ShellY:= PlayY-1;
		Armed:= false;
		if Field(ShellX, ShellY) # Blank then
			[ScoreHit;  Armed:= true]
		else SetUp(ShellX, ShellY, Shell, 0);
		];
	]
else	[					\shell is in the air
	ScoreHit;				\did something move into shell?
	SetUp(ShellX, ShellY, Blank, 0);	\remove shell from old location
	if ShellY < 1 then Armed:= true		\off top of screen--missed
	else	[ShellY:= ShellY -1;		\move shell up
		if Field(ShellX, ShellY) # Blank then \it hit something
			[ScoreHit;  Armed:= true]
		else SetUp(ShellX, ShellY, Shell, 0);
		];
	];
];	\MoveShell

\-------------------------------------------------------------------------------

proc	MovePlayer;		\Move the player
int	MX, MY, X, Y, S;
[if Bitten then return;		\but don't move bitten player

\Make tenative move of one character cell toward mouse pointer
MX:= GetMousePosition(0)*Width/320;		\(0 thru Width-1)
X:= PlayX;					\make tenative move toward mouse
if MX>PlayX & PlayX<Right then X:= PlayX+1;
if MX<PlayX & PlayX>0     then X:= PlayX-1;

MY:= TopLimit + GetMousePosition(1)*5/200;	\(0 thru 4)
Y:= PlayY;					\BotLimit-TopLimit+1 = 5
if MY>PlayY & PlayY<BotLimit then Y:= PlayY+1;
if MY<PlayY & PlayY>TopLimit then Y:= PlayY-1;

S:= Field(X, Y);				\get contents of target location
if S=Head ! S=Segment ! S=SpiderL ! S=SpiderR ! S=Flea then Bitten:= true;
if S#Mush & S#Mush1 & S#Mush2 & S#Mush3 then	\don't move over mushrooms
	[SetUp(PlayX, PlayY, Blank, 0);		\remove player from old position
	 SetUp(X, Y, Player, 0);		\display player in new position
	PlayX:= X;  PlayY:= Y;			\save player's coordinates
	]
else	MoveMouse(PlayX*8, (PlayY-TopLimit)*40);
   \move mouse position to X,Y to remove springiness (assumes Bottom = Height-1)
];	\MovePlayer

\-------------------------------------------------------------------------------

proc	PlayRound;		\Play one round (until player is bitten & dies)
int	SegXTbl, SegXDirTbl,
	I, Looper,		\makes some things happen faster than others
	Beat,			\centipede footstep sound
	ModWave;		\wave number, modulo 12 (0..11)
[\Initialize centipede segment arrays: Table(ModWave, Segment)
\ (assumes Length = 12 and Width = 40)
\		Segment (incl. head) -->		      ModWave
SegXTbl:=	\					      -------
      [ [28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39],	\0
	[27, 10,  9,  8,  7,  6,  5,  4,  3,  2,  1,  0],	\1
	[10, 18, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39],	\2
	[35, 24, 12,  8,  7,  6,  5,  4,  3,  2,  1,  0],	\3
	[ 8, 17, 24, 29, 32, 33, 34, 35, 36, 37, 38, 39],	\4
	[34, 27, 20, 14,  9,  6,  5,  4,  3,  2,  1,  0],	\5
	[ 5, 12, 19, 24, 27, 32, 34, 35, 36, 37, 38, 39],	\6
	[34, 28, 23, 18, 15, 10,  7,  4,  3,  2,  1,  0],	\7
	[ 0,  5, 10, 15, 20, 24, 28, 32, 36, 37, 38, 39],	\8
	[37, 32, 27, 23, 19, 15, 11,  8,  5,  2,  1,  0],	\9
	[ 0,  4,  8, 11, 14, 17, 22, 27, 30, 34, 38, 39],	\10
	[39, 34, 30, 27, 24, 20, 18, 14, 10,  7,  3,  0] ];	\11
SegXDirTbl:=
      [ [-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1],	\0
	[-1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1],	\1
	[-1,  1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1],	\2
	[-1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1],	\3
	[ 1, -1, -1,  1, -1, -1, -1, -1, -1, -1, -1, -1],	\4
	[ 1,  1, -1,  1,  1,  1,  1,  1,  1,  1,  1,  1],	\5
	[ 1, -1,  1, -1,  1, -1, -1, -1, -1, -1, -1, -1],	\6
	[ 1, -1, -1,  1,  1,  1, -1, -1,  1,  1,  1,  1],	\7
	[ 1,  1,  1, -1, -1, -1,  1, -1, -1, -1, -1, -1],	\8
	[ 1,  1,  1,  1, -1,  1,  1, -1, -1,  1,  1,  1],	\9
	[ 1,  1, -1, -1, -1,  1,  1,  1, -1,  1, -1, -1],	\10
	[-1, -1, -1, -1, -1, -1, -1,  1,  1,  1,  1,  1] ];	\11

ShowPlayers;					\show remaining players

PlayX:= Center;  PlayY:= BotLimit;		\set up player
MoveMouse(PlayX*8, (PlayY-TopLimit)*40);
\move mouse position to remove springiness (assumes Bottom = Height-1)

SetUp(PlayX, PlayY, Player, 0);
Bitten:= false;
Armed:= true;

StartSpider;
SpiLimit:= SpiLimIni;

FleaDrop:= false;				\don't drop flea yet
Beat:= 0;

repeat	Cursor(0, Bottom);  Attrib(Yellow);  IntOut(6, Wave+1);

	ModWave:= Rem(Wave/12);				\hatch centipede
	for I:= 0 to Tail do
		[SegX(I):= SegXTbl(ModWave, I);
		 SegY(I):= 0;				\top row
		SegXDir(I):= SegXDirTbl(ModWave, I);
		SegYDir(I):= 1;				\move downward
		Seg(I):= if I <= ModWave then Head else Segment;
		];
	for I:= 0 to Tail do SetUp(SegX(I), SegY(I), Seg(I), SegXDir(I)<0);
	SegCnt:= Length;
	HitBottom:= false;
	FullMoon:= 0;

	\Loop until centipede is dead
	loop	[SpiLimit:= SpiLimIni + Score/10000;
		if SpiLimit > TopLimit then SpiLimit:= TopLimit;

		I:= SpeedIni - Score/10000; \speed up game as score increases
		if I < SpeedMax then I:= SpeedMax;
		for Looper:= 0 to I do
			[		\this loop is run 60 times per second
			Delay(1);	\it's the surrounding loop that varies
			MovePlayer;	\ in speed according to Score/10000
			MoveShell;
			if Bitten then [KillPlayer;  quit];
			if SegCnt <= 0 then
				[Wave:= Wave+1;  CycleColors;  quit];
			if Looper = I/2 then
				[if SegCnt = 1 then MoveCent;
				if FleaMad then MoveFlea;
				if Score/10000 then MoveSpider;
				];
			];
		MoveCent;
		Beat:= Beat+1;
		if Beat >= 2 then
			[PlaySound("Centipede_Beat.wav");
			Beat:= 0;
			];
		InjectHead;
		MoveSpider;
		MoveFlea;
		];
until	Bitten;

\Clean up the carnage
for I:= 0 to Tail do				\remove centipede
    if Seg(I) # Blank then
	SetUp(SegX(I), SegY(I), Blank, 0);

if not Armed then				\remove shell from flight
	SetUp(ShellX, ShellY, Blank, 0);

if FleaDrop then				\remove flea
	SetUp(FleaX, FleaY, Blank, 0);

SetUp(SpiX, SpiY, Blank, 0);			\remove spider
SetUp(SpiX+1, SpiY, Blank, 0);

SetUp(PlayX, PlayY, Blank, 0);			\remove player
];	\PlayRound

\-------------------------------------------------------------------------------

proc	LoadSprites;		\Load sprite images from .bmp file
int	Hand,			\file handle
	Width, Height,		\dimensions of entire image in .bmp file, pixels
	R, G, B,		\red, green, blue
	X, Y,			\screen coordinates (pixels)
	I, T;			\index and tempory scratch

	proc	LoadSp(X0, Y0, W, H, Sp);	\Load a sprite
	int	X0, Y0,		\coordinates in Screen to get sprite from
		W, H;		\width and height (pixels)
	char	Sp;		\sprite to load
	int	K, X, Y;
	[Sp(0):= W;
	 Sp(1):= H;
	K:= 2;
	for Y:= Y0 to Y0+H-1 do
	    for X:= X0 to X0+W-1 do
		[Sp(K):= ReadPix(X, Y);
		K:= K+1;
		];
	];	\LoadSp

[\Read in a 256-color .bmp file
Trap(false);			\don't trap "file not found" error; we'll do it
Hand:= FOpen("pede.bmp", 0);	\open file for input
if GetErr \#0\ then
	[SetVid(3);
	Text(0, "pede.bmp file not found
");
	exit;
	];
Trap(true);			\turn traps back on
FSet(Hand, ^I);			\set device 3 to handle
OpenI(3);

for Y:= 0 to 17 do X:= ChIn(3);	\skip unused header info
Width:= ChIn(3) + ChIn(3)<<8;	\0..32764 (ample range)
for Y:= 0 to 2-1 do X:= ChIn(3);\skip
Height:= ChIn(3) + ChIn(3)<<8;	\0..32767 (ample range)
for Y:= 24 to 53 do X:= ChIn(3);\skip

for I:= 0 to 255 do		\load Palette from .bmp file
	[B:= ChIn(3);
	 G:= ChIn(3);
	 R:= ChIn(3);
	 T:= ChIn(3);
	Palette(I*3+0):= R;
	Palette(I*3+1):= G;
	Palette(I*3+2):= B;
	];
for I:= 0 to 255 do		\set color registers from Palette
	SetPalette(I, Palette(I*3+0), Palette(I*3+1), Palette(I*3+2));

\Load .bmp image onto the screen
for Y:= Height-1 downto 0 do	\.bmp files are upside down!
	[for X:= 0 to Width-1 do
		Point(X, Y, ChIn(3));
	];
FClose(Hand);			\close handle so it can be used again

\			 X,  Y, W, H, Sprite
for I:= 0 to 7 do LoadSp(8*I,  0, 8, 8, @SpSeg(I, 0));
for I:= 0 to 7 do LoadSp(8*I,  8, 8, 8, @SpHead(I, 0));
LoadSp(32, 24, 8, 8, SpBlank);
LoadSp( 0, 16, 8, 8, SpPlay);
LoadSp( 8, 16, 8, 8, SpShell);
LoadSp( 0, 24, 8, 8, SpMush);
LoadSp( 8, 24, 8, 8, SpMush3);
LoadSp(16, 24, 8, 8, SpMush2);
LoadSp(24, 24, 8, 8, SpMush1);
LoadSp( 0, 32, 8, 8, SpSpiL0);
LoadSp( 8, 32, 8, 8, SpSpiR0);
LoadSp(16, 32, 8, 8, SpSpiL1);
LoadSp(24, 32, 8, 8, SpSpiR1);
LoadSp( 0, 40, 8, 8, SpFlea0);
LoadSp( 8, 40, 8, 8, SpFlea1);
LoadSp( 0, 48, 8, 8, SpExplo0);
LoadSp( 8, 48, 8, 8, SpExplo1);
];	\LoadSprites

\-------------------------------------------------------------------------------

proc	PlayGame;		\Play one game (until all players die)
int	X, Y, I;
[LoadSprites;			\load sprites and set up color registers

\Show introductory screen
Clear;
X:= Center -15;  Y:= Height/2 -9;
Attrib(Yellow);
Cursor(Center-16, Y+8);   Text(6, " New player every 10000 points ");
Attrib(LRed);
Cursor(Center-16, Y+11);  Text(6, "     Hit Esc to exit game      ");
Attrib(LCyan);
Cursor(Center-16, Y+13);  Text(6, "     S key controls sound      ");
Attrib(LGreen);
Cursor(Center-16, Y+15);  Text(6, "     Click mouse to start      ");

\Show results of previous game
Cursor(5, Bottom-1);  Attrib(White);  Text(6, "Wave");
Cursor(5, Bottom);  Attrib(Yellow);  IntOut(6, Wave+1);
Cursor(Width-23, Bottom-1);  Attrib(White);  Text(6, "Score ");
Cursor(Width-23, Bottom);    Attrib(Yellow);  IntOut(6, Score);
Cursor(Width-10, Bottom-1);  Attrib(White);  Text(6, "High ");
Cursor(Width-10, Bottom);    Attrib(Yellow);  IntOut(6, HiScore);

\Wiggle the cute little centipede feet
loop	[for I:=  0 to 12 do DrawObj(X+I, Y, Segment, 0);
	DrawObj(X+13, Y, Head, 0);
	DrawObj(X+15, Y, Head, 1);
	for I:= 16 to 28 do DrawObj(X+I, Y, Segment, 1);

	DrawObj(X, Y+1, Segment, 0);  DrawObj(X+28, Y+1, Segment, 1);
	DrawObj(X, Y+2, Segment, 0);
	Cursor(X+1, Y+2);  Attrib(BWhite);
	Text(6, "  P E D E I C I D E   1.0  ");
	DrawObj(X+28, Y+2, Segment, 1);
	DrawObj(X, Y+3, Segment, 0);  DrawObj(X+28, Y+3, Segment, 1);

	for I:=  0 to 13 do DrawObj(X+I, Y+4, Segment, 0);
	for I:= 14 to 28 do DrawObj(X+I, Y+4, Segment, 1);

	for I:= 0 to 9 do
		[Delay(1);
		if GetMouseButton then quit;	\click mouse...
		];
	];

\Unveil screen full of mushooms
for Y:= 0 to BotLimit-1 do
    [for X:= 0 to Width-1 do
	SetUp(X, Y, if Ran(Width*BotLimit) > MushIni then Blank else Mush, 0);
    Delay(2);
    ];
for X:= 0 to Width-1 do
    SetUp(X, BotLimit, Blank, 0);

Wave:= 0;
Score:= 0;
Players:= PlayIni;

ShowPlayers;					\show the waiting players

\Show the scores
Cursor(Width-24, Bottom);  Attrib(White);  Text(6, "Score ");
Attrib(Yellow);  IntOut(6, Score);
Cursor(Width-11, Bottom);  Attrib(White);  Text(6, "High ");
Attrib(Yellow);  IntOut(6, HiScore);

while Players > 0 do PlayRound;

if Score > HiScore then HiScore:= Score;

for II:= 1 to 12 do Noise(9, 227+19*II);	\depressing sound--you lose!
Delay(60);

while GetMouseButton do [];			\wait until buttons are released
];	\PlayGame

\===============================================================================

[if not OpenMouse then
	[Text(0, "This program requires a mouse.");  exit];

SetVid($13);			\320x200 graphics in 256 colors
Vol:= true;
Wave:= 0;			\(PlayGame shows these initial values)
Score:= 0;
HiScore:= 0;

loop PlayGame;			\(Esc key exits program)
]
